@@ -14,6 +14,7 @@ from pprint import pprint
 import re
 import sys
 import time
+import traceback
 import warnings
 try:                    # Python 3
     import configparser
@@ -60,8 +61,9 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '1.1'
-__all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service', 'read_config']
+__version__ = '1.2'
+__all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
+           'format_exception', 'read_config']
 
 CONF_FILE = 'erppeek.ini'
 HIST_FILE = os.path.expanduser('~/.erppeek_history')
@@ -112,8 +114,8 @@ DOMAIN_OPERATORS = frozenset('!|&')
 #  - redundant operators: '<>', '=like', '=ilike'
 #  - future operator(s) (6.1): '=?'
 _term_re = re.compile(
-    '(\S+)\s*'
-    '(=|!=|>|>=|<|<=|like|ilike|in|not like|not ilike|not in|child_of)'
+    '([\w._]+)\s*'
+    '(=|!=|>|>=|<|<=|(?<= )(?:like|ilike|in|not like|not ilike|not in|child_of))'
     '\s*(.*)')
 _fields_re = re.compile(r'(?:[^%]|^)%\(([^)]+)\)')
 
@@ -170,6 +172,25 @@ def lowercase(s, _sub=re.compile('[A-Z]').sub,
         return s
 
 
+def format_exception(exc_type, exc, tb, limit=None, chain=True,
+                     _format_exception=traceback.format_exception):
+    """Format a stack trace and the exception information.
+
+    This wrapper is a replacement of ``traceback.format_exception``
+    which formats the error and traceback received by XML-RPC.
+    """
+    # Format readable 'Fault' errors
+    if ((issubclass(exc_type, Fault) and
+         isinstance(exc.faultCode, basestring))):
+        etype, _, msg = exc.faultCode.partition('--')
+        if etype.strip() != 'warning':
+            msg = exc.faultCode
+            if not msg.startswith('FATAL:'):
+                msg += '\n' + exc.faultString
+        return ['%s: %s\n' % (exc_type.__name__, msg.strip())]
+    return _format_exception(exc_type, exc, tb, limit=limit)
+
+
 def read_config(section=None):
     """Read the environment settings from the configuration file.
 
@@ -186,14 +207,10 @@ def read_config(section=None):
         p.readfp(f)
     if section is None:
         return p.sections()
-    server = 'http://%s:%s' % (p.get(section, 'host'), p.get(section, 'port'))
-    db = p.get(section, 'database')
-    user = p.get(section, 'username')
-    if p.has_option(section, 'password'):
-        password = p.get(section, 'password')
-    else:
-        password = None
-    return (server, db, user, password)
+    env = dict(p.items(section))
+    scheme = env.get('scheme', 'http')
+    server = '%s://%s:%s' % (scheme, env['host'], env['port'])
+    return (server, env['database'], env['username'], env.get('password'))
 
 
 def issearchdomain(arg):
@@ -608,7 +625,10 @@ class Client(object):
         mods = self.read('ir.module.module',
                          [('state', 'not in', STABLE_STATES)], 'name state')
         if not mods:
-            print('%s module(s)' % updated)
+            if modules:
+                print('Module(s) not found: %s' % ', '.join(modules))
+            else:
+                print('%s module(s) updated' % updated)
             return
         print('%s module(s) selected' % len(ids))
         print('%s module(s) to process:' % len(mods))
@@ -1175,10 +1195,8 @@ class Record(object):
                 kwargs.setdefault('context', context)
             res = self.client.execute(
                 self._model_name, attr, [self.id], *params, **kwargs)
-            try:
-                (res,) = res
-            except Exception:
-                pass
+            if isinstance(res, list) and len(res) == 1:
+                return res[0]
             return res
         wrapper.__name__ = attr
         wrapper.__doc__ %= (self._model_name, attr, self.id)
@@ -1207,19 +1225,6 @@ def _interact(use_pprint=True, usage=USAGE):
     # Do not run twice
     del globals()['_interact']
 
-    def excepthook(exc_type, exc, tb, _original_hook=sys.excepthook):
-        # Print readable 'Fault' errors
-        if ((issubclass(exc_type, Fault) and
-             isinstance(exc.faultCode, basestring))):
-            etype, _, msg = exc.faultCode.partition('--')
-            if etype.strip() != 'warning':
-                msg = exc.faultCode
-                if not msg.startswith('FATAL:'):
-                    msg += '\n' + exc.faultString
-            print('%s: %s' % (exc_type.__name__, msg.strip()))
-        else:
-            _original_hook(exc_type, exc, tb)
-
     if use_pprint:
         def displayhook(value, _printer=pprint, _builtins=builtins):
             # Pretty-format the output
@@ -1257,8 +1262,10 @@ def _interact(use_pprint=True, usage=USAGE):
             except SystemExit:
                 raise
             except:
+                # Print readable 'Fault' errors
                 # Work around http://bugs.python.org/issue12643
-                excepthook(*sys.exc_info())
+                msg = ''.join(format_exception(*sys.exc_info()))
+                print(msg.strip())
 
     warnings.simplefilter('always', UserWarning)
     # Key UP to avoid an empty line
