@@ -3,7 +3,7 @@ import mock
 from mock import sentinel, ANY
 
 import erppeek
-from ._common import XmlRpcTestCase, OBJ
+from ._common import XmlRpcTestCase, OBJ, callable
 
 
 class TestCase(XmlRpcTestCase):
@@ -28,7 +28,7 @@ class TestCase(XmlRpcTestCase):
                     self._id = id_
 
                 def __getitem__(self, key):
-                    if key is 'id':
+                    if key == 'id':
                         return self._id
                     return 'v_' + key
             if isinstance(args[5], int):
@@ -383,29 +383,87 @@ class TestRecord(TestCase):
         )
         self.assertOutput('')
 
-    def test_method(self, method_name='method', single_id=True):
+    def test_method(self, method_name='method'):
         records = self.model('foo.bar').browse([13, 17])
         rec = self.model('foo.bar').browse(42)
         records_method = getattr(records, method_name)
         rec_method = getattr(rec, method_name)
 
-        single_id = single_id and 42 or [42]
-
         records_method()
         rec_method()
         self.assertCalls(
             OBJ('foo.bar', 'fields_get_keys'),
             OBJ('foo.bar', method_name, [13, 17]),
-            OBJ('foo.bar', method_name, single_id),
+            OBJ('foo.bar', method_name, [42]),
         )
         self.assertOutput('')
 
     def test_standard_methods(self):
-        for method in 'create browse'.split():
-            self.test_method(method)
-            del self.model('foo.bar')._keys
-
         # write copy
-        self.test_method('unlink', single_id=False)
+        self.test_method('unlink')
         del self.model('foo.bar')._keys
-        self.test_method('perm_read', single_id=False)
+        self.test_method('perm_read')
+
+    def test_empty_recordlist(self):
+        records = self.model('foo.bar').browse([13, 17])
+        empty = records[42:]
+
+        self.assertIsInstance(records, erppeek.RecordList)
+        self.assertTrue(records)
+        self.assertEqual(len(records), 2)
+        self.assertEqual(records.name, ['v_name'] * 2)
+
+        self.assertIsInstance(empty, erppeek.RecordList)
+        self.assertFalse(empty)
+        self.assertEqual(len(empty), 0)
+        self.assertEqual(empty.name, [])
+
+        self.assertCalls(
+            OBJ('foo.bar', 'fields_get_keys'),
+            OBJ('foo.bar', 'read', [13, 17], ['name']),
+            OBJ('foo.bar', 'fields_get'),
+        )
+        self.assertOutput('')
+
+    def test_attr(self):
+        records = self.model('foo.bar').browse([13, 17])
+        rec = self.model('foo.bar').browse(42)
+
+        # existing fields can be read as attributes
+        # attribute is writable on the Record object only
+        self.assertFalse(callable(rec.message))
+        rec.message = 'one giant leap for mankind'
+        self.assertFalse(callable(rec.message))
+        self.assertEqual(records.message, ['v_message', 'v_message'])
+
+        # if the attribute is not a field, it could be a specific RPC method
+        self.assertEqual(rec.missingattr(), sentinel.OTHER)
+        self.assertEqual(records.missingattr(), [sentinel.OTHER])
+
+        self.assertCalls(
+            OBJ('foo.bar', 'fields_get_keys'),
+            OBJ('foo.bar', 'read', 42, ['message']),
+            OBJ('foo.bar', 'write', [42], {'message': 'one giant leap for mankind'}),
+            OBJ('foo.bar', 'read', 42, ['message']),
+            OBJ('foo.bar', 'read', [13, 17], ['message']),
+            OBJ('foo.bar', 'fields_get'),
+            OBJ('foo.bar', 'missingattr', [42]),
+            OBJ('foo.bar', 'missingattr', [13, 17]),
+        )
+
+        # `setattr` not allowed (except for existing fields on Record object)
+        self.assertRaises(AttributeError, setattr, rec, 'missingattr', 42)
+        self.assertRaises(AttributeError, setattr, records, 'message', 'one')
+        self.assertRaises(AttributeError, setattr, records, 'missingattr', 42)
+
+        # method can be forgotten (any use case?)
+        del rec.missingattr, records.missingattr
+
+        # `del` not allowed for attributes or missing attr
+        self.assertRaises(AttributeError, delattr, rec, 'message')
+        self.assertRaises(AttributeError, delattr, records, 'message')
+        self.assertRaises(AttributeError, delattr, rec, 'missingattr2')
+        self.assertRaises(AttributeError, delattr, records, 'missingattr2')
+
+        self.assertCalls()
+        self.assertOutput('')
